@@ -172,7 +172,7 @@ read_channel_handle_request (GVfsChannel *channel,
     default:
       g_set_error (error, G_IO_ERROR,
 		   G_IO_ERROR_FAILED,
-		   "Unknown stream command %"G_GINT32_FORMAT"\n", command);
+		   "Unknown stream command %"G_GUINT32_FORMAT, command);
       break;
     }
 
