@@ -136,6 +136,54 @@ changed_in_idle (gpointer data)
   return FALSE;
 }
 
+#define KILOBYTE_FACTOR 1000.0
+#define MEGABYTE_FACTOR (1000.0 * 1000.0)
+#define GIGABYTE_FACTOR (1000.0 * 1000.0 * 1000.0)
+
+/**
+ * format_size_for_display:
+ * @size: a number of octects
+ *
+ * Format a human readable string that can conveys how much storage a
+ * user-visible drive or piece of media can hold.
+ *
+ * As a matter of policy, we want this string to resemble what's on
+ * the packaging of the drive/media. Since all manufacturers use
+ * powers of 10, g_format_size_for_display() is not suitable here.
+ *
+ * TODO: we probably want to round to nearest power of two if @size is
+ * "close" (e.g. within 5%) - this is to avoid e.g. 63.4G when the
+ * packaging says "64G drive". We could also use per-drive or
+ * per-media quirks to make a better guess.
+ *
+ * Returns: A human readable string, caller must free it using
+ * g_free().
+ **/
+static char *
+format_size_for_display (guint64 size)
+{
+  char *str;
+  gdouble displayed_size;
+  
+  if (size < MEGABYTE_FACTOR)
+    {
+      displayed_size = (double) size / KILOBYTE_FACTOR;
+      str = g_strdup_printf (_("%.1f kB"), displayed_size);
+    } 
+  else if (size < GIGABYTE_FACTOR)
+    {
+      displayed_size = (double) size / MEGABYTE_FACTOR;
+      str = g_strdup_printf (_("%.1f MB"), displayed_size);
+    } 
+  else 
+    {
+      displayed_size = (double) size / GIGABYTE_FACTOR;
+      str = g_strdup_printf (_("%.1f GB"), displayed_size);
+    }
+  
+  return str;
+}
+
 static void
 do_update_from_hal (GHalVolume *mv)
 {
@@ -187,7 +235,7 @@ do_update_from_hal (GHalVolume *mv)
     {
       if (strcmp (volume_fsusage, "crypto") == 0 && strcmp (volume_fstype, "crypto_LUKS") == 0)
         {
-          size = g_format_size_for_display (volume_size);
+          size = format_size_for_display (volume_size);
           /* Translators: %s is the size of the volume (e.g. 512 MB) */
           name = g_strdup_printf (_("%s Encrypted Data"), size);
           g_free (size);
@@ -210,7 +258,7 @@ do_update_from_hal (GHalVolume *mv)
             }
           else
             {
-              size = g_format_size_for_display (volume_size);
+              size = format_size_for_display (volume_size);
               /* Translators: %s is the size of the volume (e.g. 512 MB) */
               name = g_strdup_printf (_("%s Media"), size);
               g_free (size);
@@ -712,9 +760,10 @@ spawn_cb (GPid pid, gint status, gpointer user_data)
   GSimpleAsyncResult *simple;
 
   /* ensure that the #GHalMount corrosponding to the #GHalVolume we've
-   * mounted is made available before returning to the user
+   * mounted is made available before returning to the user (make sure
+   * we don't emit the signals in idle; see #552168).
    */
-  g_hal_volume_monitor_force_update (G_HAL_VOLUME_MONITOR (G_HAL_VOLUME (data->object)->volume_monitor));
+  g_hal_volume_monitor_force_update (G_HAL_VOLUME_MONITOR (G_HAL_VOLUME (data->object)->volume_monitor), FALSE);
 
   if (WEXITSTATUS (status) != 0)
     {
